@@ -10,29 +10,32 @@ class VisualTSV(tsv.TSV):
     def __init__(self, header, data):
         super().__init__(header, data)
 
-    def linechart(self, xcol, ycols, ylabel = None, title = None, subplots = False, xfigsize = 25, yfigsize = 5):
-        return __pd_linechart__(self, xcol, ycols, ylabel, title, subplots, xfigsize, yfigsize)
+    def linechart(self, xcol, ycols, ylabel = None, title = None, subplots = False, xfigsize = 25, yfigsize = 5, props = None):
+        return __pd_linechart__(self, xcol, ycols, ylabel, title, subplots, xfigsize, yfigsize, props)
 
-    def scatterplot(self, xcol, ycol, class_col = None, title = None, xfigsize = 25, yfigsize = 5, max_rows = 20, max_class_col = 10):
-        return __sns_scatterplot__(self, xcol, ycol, class_col, title, xfigsize, yfigsize, max_rows, max_class_col)
+    def scatterplot(self, xcol, ycol, class_col = None, title = None, xfigsize = 25, yfigsize = 5, max_rows = 20, max_class_col = 10, props = None):
+        return __sns_scatterplot__(self, xcol, ycol, class_col, title, xfigsize, yfigsize, max_rows, max_class_col, props)
 
-    def histogram(self, xcol, class_col = None, bins = 10, title = None, binwidth = None, kde = False, multiple = "dodge", xfigsize = 25, yfigsize = 5, max_class_col = 10):
-        return __sns_histogram__(self, xcol, class_col, bins, title, binwidth, kde, multiple, xfigsize, yfigsize, max_class_col)
+    def histogram(self, xcol, class_col = None, bins = 10, title = None, binwidth = None, xfigsize = 25, yfigsize = 5, max_class_col = 10, props = None):
+        return __sns_histogram__(self, xcol, class_col, bins, title, binwidth, xfigsize, yfigsize, max_class_col, props)
 
-    def density(self, ycols, class_col = None, xfigsize = 25, yfigsize = 5):
-        return __sns_density__(self, ycols, class_col, xfigsize, yfigsize)
+    def ecdf(self, xcol, class_col = None, title = None, xfigsize = 25, yfigsize = 5, max_class_col = 10, props = None):
+        return __sns_ecdf__(self, xcol, class_col, title, xfigsize, yfigsize, max_class_col, props)
 
-    def barchart(self, xcol, ycol, class_col = None, resort = True, xfigsize = 25, yfigsize = 5, max_rows = 20, max_class_col = 10):
-        return __sns_barplot__(self, xcol, ycol, class_col, resort, xfigsize, yfigsize, max_rows, max_class_col)
+    def density(self, ycols, class_col = None, xfigsize = 25, yfigsize = 5, props = None):
+        return __sns_density__(self, ycols, class_col, xfigsize, yfigsize, props)
 
-    def boxplot(self, xcol, ycol, class_col = None, xfigsize = 25, yfigsize = 5, max_rows = 20, max_class_col = 10):
-        return __sns_boxplot__(self, xcol, ycol, class_col, xfigsize, yfigsize, max_rows, max_class_col)
+    def barchart(self, xcol, ycol, class_col = None, xfigsize = 25, yfigsize = 5, max_rows = 20, max_class_col = 10, props = None):
+        return __sns_barplot__(self, xcol, ycol, class_col, xfigsize, yfigsize, max_rows, max_class_col, props)
 
-    def corr_heatmap(self, cols, xfigsize = 25, yfigsize = 5, max_rows = 6):
-        return __sns_corr_heatmp__(self, cols, xfigsize, yfigsize, max_rows)
+    def boxplot(self, xcol, ycol, class_col = None, xfigsize = 25, yfigsize = 5, max_rows = 20, max_class_col = 10, props = None):
+        return __sns_boxplot__(self, xcol, ycol, class_col, xfigsize, yfigsize, max_rows, max_class_col, props)
 
-    def pairplot(self, cols, class_col = None, kind = None, diag_kind = None, xfigsize = 5, yfigsize = 5, max_rows = 6, max_class_col = 6):
-        return __sns_pairplot__(self, cols, class_col, kind, diag_kind, xfigsize, yfigsize, max_rows, max_class_col)
+    def corr_heatmap(self, cols, xfigsize = 25, yfigsize = 5, max_rows = 6, props = None):
+        return __sns_corr_heatmp__(self, cols, xfigsize, yfigsize, max_rows, props)
+
+    def pairplot(self, cols, class_col = None, xfigsize = 5, yfigsize = 5, max_rows = 6, max_class_col = 6, props = None):
+        return __sns_pairplot__(self, cols, class_col, xfigsize, yfigsize, max_rows, max_class_col, props)
 
 def __create_data_frame_with_types__(xtsv, xcol = None, ycols = None, zcol = None):
     # convert to array
@@ -67,7 +70,29 @@ def __create_data_frame_with_types__(xtsv, xcol = None, ycols = None, zcol = Non
 
     return pd.DataFrame(mp)
 
-def __pd_linechart__(xtsv, xcol, ycols, ylabel, title, subplots, xfigsize, yfigsize):
+def __merge_props__(props, default_props):
+    # create new dict 
+    props2 = dict()
+
+    # take new props
+    if (props is not None):
+        for k in props.keys():
+            props2[k] = props[k]
+
+    # take default if not defined
+    if (default_props is not None):
+        for k in default_props.keys():
+            if (k not in props2.keys()):
+                props2[k] = default_props[k]
+
+    # return
+    return props2
+
+def __pd_linechart__(xtsv, xcol, ycols, ylabel, title, subplots, xfigsize, yfigsize, props):
+    # default props
+    default_props = dict()
+    props2 = __merge_props__(props, default_props)
+
     # validate ycols
     ycols = xtsv.__get_matching_cols__(ycols)
 
@@ -86,12 +111,16 @@ def __pd_linechart__(xtsv, xcol, ycols, ylabel, title, subplots, xfigsize, yfigs
     df = __create_data_frame_with_types__(xtsv, xcol, ycols, None)
 
     # plot
-    df.plot.line(subplots = subplots, x = xcol, ylabel = ylabel, figsize = (xfigsize, yfigsize), title = title)
+    df.plot.line(subplots = subplots, x = xcol, ylabel = ylabel, figsize = (xfigsize, yfigsize), title = title, **props2)
 
     # return
     return VisualTSV(xtsv.get_header(), xtsv.get_data())
 
-def __sns_scatterplot__(xtsv, xcol, ycol, class_col, title, xfigsize, yfigsize, max_rows, max_class_col):
+def __sns_scatterplot__(xtsv, xcol, ycol, class_col, title, xfigsize, yfigsize, max_rows, max_class_col, props):
+    # default props
+    default_props = dict()
+    props2 = __merge_props__(props, default_props)
+
     # check number of unique class values
     if (class_col is not None and len(xtsv.col_as_array_uniq(class_col)) >= max_class_col):
         raise Exception("Number of class column values is more than {}: {}. Probably not a class column. Try max_class_col".format(max_class_col, len(xtsv.col_as_array_uniq(class_col))))
@@ -116,54 +145,118 @@ def __sns_scatterplot__(xtsv, xcol, ycol, class_col, title, xfigsize, yfigsize,
     if (title is None):
         title = "{} vs {}".format(xcol, ycol)
 
-    #df.plot.scatter(x = xcol, y = ycol, figsize = figsize, title = title)
+    # take hue order
+    hue_order = sorted(xtsv.col_as_array_uniq(class_col)) if (class_col is not None) else None
+
+    # plot
     ax.set_title(title)
-    sns.scatterplot(ax = ax, x = xcol, y = ycol, hue = class_col, data = df)
+    sns.scatterplot(ax = ax, x = xcol, y = ycol, hue = class_col, hue_order = hue_order, data = df, **props)
 
     # return
     return VisualTSV(xtsv.get_header(), xtsv.get_data())
 
-def __sns_histogram__(xtsv, xcol, class_col, bins, title, binwidth, kde, multiple, xfigsize, yfigsize, max_class_col):
+def __sns_histogram__(xtsv, xcol, class_col, bins, title, binwidth, xfigsize, yfigsize, max_class_col, props):
+    # default props
+    default_props = dict(multiple = "dodge", shrink = 0.8, kde = False)
+    props2 = __merge_props__(props, default_props)
+
     # check number of unique class values
     if (class_col is not None and len(xtsv.col_as_array_uniq(class_col)) >= max_class_col):
         raise Exception("Number of class column values is more than {}: {}. Probably not a class column. Try max_class_col".format(max_class_col, len(xtsv.col_as_array_uniq(class_col))))
 
+    # take class col if defined
+    if (class_col is not None):
+        xtsv = xtsv.sort([class_col, xcol])
+    else:
+        xtsv = xtsv.sort(xcol)
+
+    # create dataframe
     df = __create_data_frame_with_types__(xtsv, xcol, None, class_col)
 
+    # create figure
     figsize = (xfigsize, yfigsize)
     fig, ax = pyplot.subplots(figsize = figsize)
 
-    # binwidth overrides bins
+    # take hue order
+    hue_order = sorted(xtsv.col_as_array_uniq(class_col)) if (class_col is not None) else None
+
+    # binwidth overrides bins. TODO: This hue parameter is not giving class color consistently
     if (binwidth is not None):
-        sns.histplot(data = df, x = xcol, hue = class_col, binwidth = binwidth, kde = kde, multiple = multiple, shrink = 0.8)
+        sns.histplot(data = df, x = xcol, hue = class_col, hue_order = hue_order, binwidth = binwidth, **props2)
     else:
-        sns.histplot(data = df, x = xcol, hue = class_col, bins = bins, kde = kde, multiple = multiple, shrink = 0.8)
+        sns.histplot(data = df, x = xcol, hue = class_col, hue_order = hue_order, bins = bins, **props2)
+
+    # return
+    return VisualTSV(xtsv.get_header(), xtsv.get_data())
+
+def __sns_ecdf__(xtsv, xcol, class_col, title, xfigsize, yfigsize, max_class_col, props):
+    # default props
+    default_props = dict()
+    props2 = __merge_props__(props, default_props)
+
+    # check number of unique class values
+    if (class_col is not None and len(xtsv.col_as_array_uniq(class_col)) >= max_class_col):
+        raise Exception("Number of class column values is more than {}: {}. Probably not a class column. Try max_class_col".format(max_class_col, len(xtsv.col_as_array_uniq(class_col))))
+
+    # take class col if defined
+    if (class_col is not None):
+        xtsv = xtsv.sort([class_col, xcol])
+    else:
+        xtsv = xtsv.sort(xcol)
+
+    # create dataframe
+    df = __create_data_frame_with_types__(xtsv, xcol, None, class_col)
+
+    # create figure
+    figsize = (xfigsize, yfigsize)
+    fig, ax = pyplot.subplots(figsize = figsize)
+
+    # take hue order
+    hue_order = sorted(xtsv.col_as_array_uniq(class_col)) if (class_col is not None) else None
+
+    # binwidth overrides bins. TODO: This hue parameter is not giving class color consistently
+    sns.ecdfplot(data = df, x = xcol, hue = class_col, hue_order = hue_order, **props2)
 
     # return
     return VisualTSV(xtsv.get_header(), xtsv.get_data())
 
 # the syntax is non intuitive. need to follow row major or column major. splitting by class_col is not possible
-def __sns_density__(xtsv, ycols, class_col, xfigsize, yfigsize):
+def __sns_density__(xtsv, ycols, class_col, xfigsize, yfigsize, props):
+    # default props
+    default_props = dict(multiple = "layer")
+    props2 = __merge_props__(props, default_props)
+
     # create df
     ycols = xtsv.__get_matching_cols__(ycols)
     df = __create_data_frame_with_types__(xtsv, None, ycols, class_col)
 
+    # create figure
     figsize = (xfigsize, yfigsize)
     fig, ax = pyplot.subplots(figsize = figsize)
 
+    # take hue order
+    hue_order = sorted(xtsv.col_as_array_uniq(class_col)) if (class_col is not None) else None
+
     # TODO: This is not clean
+    # multiple = props["multiple"] if (props is not None and "multiple" in props.keys()) else "layer"
+
+    # check for class col
     if (class_col is not None):
         if (len(ycols) == 1):
-            sns.kdeplot(data = df, x = ycols[0], hue = class_col, multiple = "stack")
+            sns.kdeplot(data = df, x = ycols[0], hue = class_col, hue_order = hue_order, **props2)
         else:
-            raise Exception("__sns_density__: class_col with multiple ycols is not supported")
+            raise Exception("__sns_density__: class_col with multiple ycols is not supported: {}".format(ycols))
     else:
-       sns.kdeplot(data = df, multiple = "stack")
+       sns.kdeplot(data = df, **props2)
 
     # return
     return VisualTSV(xtsv.get_header(), xtsv.get_data())
 
-def __sns_barplot__(xtsv, xcol, ycol, class_col, resort, xfigsize, yfigsize, max_rows, max_class_col):
+def __sns_barplot__(xtsv, xcol, ycol, class_col, xfigsize, yfigsize, max_rows, max_class_col, props):
+    # default props
+    default_props = dict()
+    props2 = __merge_props__(props, default_props)
+
     # check number of unique class values
     if (class_col is not None and len(xtsv.col_as_array_uniq(class_col)) >= max_class_col):
         raise Exception("Number of class column values is more than {}: {}. Probably not a class column. Try max_class_col".format(max_class_col, len(xtsv.col_as_array_uniq(class_col))))
@@ -173,26 +266,33 @@ def __sns_barplot__(xtsv, xcol, ycol, class_col, resort, xfigsize, yfigsize, max
         utils.warn("Number of categorical values on x axis is too high: {}. Doing downsampling for clean display to max_rows: {}".format(len(xtsv.col_as_array_uniq(xcol)), max_rows))
         xtsv = xtsv.sample_column_by_max_uniq_values(xcol, max_rows)
 
-    # sort the xcol
-    if (resort == True):
-        # take class col if defined
-        if (class_col is not None):
-            xtsv = xtsv.sort([class_col, xcol])
-        else:
-            xtsv = xtsv.sort(xcol)
+    # take class col if defined
+    if (class_col is not None):
+        xtsv = xtsv.sort([class_col, xcol])
+    else:
+        xtsv = xtsv.sort(xcol)
 
     # create df
     df = __create_data_frame_with_types__(xtsv, xcol, ycol, class_col)
 
+    # create figure
     figsize = (xfigsize, yfigsize)
     fig, ax = pyplot.subplots(figsize = figsize)
 
-    sns.barplot(data = df, x = xcol, y = ycol, hue = class_col)
+    # take hue order
+    hue_order = sorted(xtsv.col_as_array_uniq(class_col)) if (class_col is not None) else None
+
+    # plot
+    sns.barplot(data = df, x = xcol, y = ycol, hue = class_col, hue_order = hue_order, **props2)
 
     # return
     return VisualTSV(xtsv.get_header(), xtsv.get_data())
 
-def __sns_boxplot__(xtsv, xcol, ycol, class_col, xfigsize, yfigsize, max_rows, max_class_col):
+def __sns_boxplot__(xtsv, xcol, ycol, class_col, xfigsize, yfigsize, max_rows, max_class_col, props):
+    # default props
+    default_props = dict()
+    props2 = __merge_props__(props, default_props)
+
     # check number of unique class values
     if (class_col is not None and len(xtsv.col_as_array_uniq(class_col)) >= max_class_col):
         raise Exception("Number of class column values is more than {}: {}. Probably not a class column. Try max_class_col".format(max_class_col, len(xtsv.col_as_array_uniq(class_col))))
@@ -208,15 +308,25 @@ def __sns_boxplot__(xtsv, xcol, ycol, class_col, xfigsize, yfigsize, max_rows, m
     # create df
     df = __create_data_frame_with_types__(xtsv, xcol, ycol, class_col)
 
+    # create figure
     figsize = (xfigsize, yfigsize)
     fig, ax = pyplot.subplots(figsize = figsize)
 
-    sns.boxplot(data = df, x = xcol, y = ycol, hue = class_col)
+    # take hue order
+    hue_order = sorted(xtsv.col_as_array_uniq(class_col)) if (class_col is not None) else None
+
+    # plot 
+    sns.boxplot(data = df, x = xcol, y = ycol, hue = class_col, hue_order = hue_order, **props2)
 
     # return
     return VisualTSV(xtsv.get_header(), xtsv.get_data())
 
-def __sns_corr_heatmp__(xtsv, cols, xfigsize, yfigsize, max_rows):
+def __sns_corr_heatmp__(xtsv, cols, xfigsize, yfigsize, max_rows, props):
+    # default props
+    default_props = dict(annot = True)
+    props2 = __merge_props__(props, default_props)
+
+    # get matching cols
     cols = xtsv.__get_matching_cols__(cols)
 
     # validation for number of columns. if the number of unique values is too high, then raise exception
@@ -231,16 +341,24 @@ def __sns_corr_heatmp__(xtsv, cols, xfigsize, yfigsize, max_rows):
     # create df
     df = __create_data_frame_with_types__(xtsv, None, cols, None)
 
+    # create figure
     figsize = (xfigsize, yfigsize)
     fig, ax = pyplot.subplots(figsize = figsize)
 
-    sns.heatmap(df.corr(), annot = True)
+    # plot
+    sns.heatmap(df.corr(), **props2)
 
     # return
     return VisualTSV(xtsv.get_header(), xtsv.get_data())
 
-def __sns_pairplot__(xtsv, cols, class_col, kind, diag_kind, xfigsize, yfigsize, max_rows, max_class_col):
+def __sns_pairplot__(xtsv, cols, class_col, xfigsize, yfigsize, max_rows, max_class_col, props):
+    # default props
+    default_props = dict(kind = None, diag_kind = None)
+    props2 = __merge_props__(props, default_props)
+
+    # find matching cols
     cols = xtsv.__get_matching_cols__(cols)
+
     # check number of unique class values
     if (class_col is not None and len(xtsv.col_as_array_uniq(class_col)) >= max_class_col):
         raise Exception("Number of class column values is more than {}. Max allowed: {}. Probably not a class column. Try max_class_col".format(max_class_col, len(xtsv.col_as_array_uniq(class_col))))
@@ -257,8 +375,12 @@ def __sns_pairplot__(xtsv, cols, class_col, kind, diag_kind, xfigsize, yfigsize,
     # create df
     df = __create_data_frame_with_types__(xtsv, None, cols, class_col)
 
+    # take hue order
+    hue_order = sorted(xtsv.col_as_array_uniq(class_col)) if (class_col is not None) else None
+
+    # define aspect and plot
     aspect = xfigsize / yfigsize
-    sns.pairplot(df, hue = class_col, kind = kind, diag_kind = diag_kind, aspect = aspect, height = yfigsize)
+    sns.pairplot(df, hue = class_col, hue_order = hue_order, kind = kind, diag_kind = diag_kind, aspect = aspect, height = yfigsize, **props2)
 
     # return
     return VisualTSV(xtsv.get_header(), xtsv.get_data())
